@@ -12,8 +12,6 @@
 ## GNU General Public License for more details.
 
 from mktxp.collectors.base_collector import BaseCollector
-from mktxp.router_metric import RouterMetric
-
 
 class HealthCollector(BaseCollector):
     ''' System Health Metrics collector
@@ -23,7 +21,7 @@ class HealthCollector(BaseCollector):
         health_labels = ['voltage', 'temperature']
         health_records = router_metric.health_records(health_labels)
         if not health_records:
-            return
+            return range(0)
 
         voltage_metrics = BaseCollector.gauge_collector('system_routerboard_voltage', 'Supplied routerboard voltage', health_records, 'voltage')
         yield voltage_metrics
