@@ -12,8 +12,6 @@
 ## GNU General Public License for more details.
 
 from mktxp.collectors.base_collector import BaseCollector
-from mktxp.router_metric import RouterMetric
-
 
 class IdentityCollector(BaseCollector):
     ''' System Identity Metrics collector
@@ -23,7 +21,7 @@ class IdentityCollector(BaseCollector):
         identity_labels = ['name']
         identity_records = router_metric.identity_records(identity_labels)
         if not identity_records:
-            return
+            return range(0)
 
         identity_metrics = BaseCollector.info_collector('system_identity', 'System identity', identity_records, identity_labels)
         yield identity_metrics
